@@ -1,16 +1,19 @@
 from __future__ import annotations
 
 from asyncio import Lock
+from functools import partial
 from types import ModuleType
 from typing import Any, Dict, Iterator, List, Optional, Tuple
 
 import firebolt.async_db as async_dbapi
 from firebolt.async_db import Connection
+from sqlalchemy.engine import AdaptedConnection  # type: ignore[attr-defined]
 
 # Ignoring type since sqlalchemy-stubs doesn't cover AdaptedConnection
 # and util.concurrency
-from sqlalchemy.engine import AdaptedConnection  # type: ignore[attr-defined]
+from sqlalchemy.pool import AsyncAdaptedQueuePool  # type: ignore[attr-defined]
 from sqlalchemy.util.concurrency import await_only  # type: ignore[import]
+from trio import run
 
 from firebolt_db.firebolt_dialect import FireboltDialect
 
@@ -126,7 +129,7 @@ class AsyncConnectionWrapper(AdaptedConnection):
         self._connection.commit()
 
     def close(self) -> None:
-        self.await_(self._connection._aclose())
+        self.await_(self._connection.aclose())
 
 
 class AsyncAPIWrapper(ModuleType):
@@ -150,8 +153,10 @@ class AsyncAPIWrapper(ModuleType):
             setattr(self, name, getattr(self.dbapi, name))
 
     def connect(self, *arg: Any, **kw: Any) -> AsyncConnectionWrapper:
-
-        connection = await_only(self.dbapi.connect(*arg, **kw))  # type: ignore[attr-defined] # noqa: F821,E501
+        # Synchronously establish a connection that can execute
+        # asynchronous queries later
+        conn_func = partial(self.dbapi.connect, *arg, **kw)  # type: ignore[attr-defined] # noqa: F821,E501
+        connection = run(conn_func)
         return AsyncConnectionWrapper(
             self,
             connection,
@@ -163,6 +168,7 @@ class AsyncFireboltDialect(FireboltDialect):
     supports_statement_cache: bool = False
     supports_server_side_cursors: bool = False
     is_async: bool = True
+    poolclass = AsyncAdaptedQueuePool
 
     @classmethod
     def dbapi(cls) -> AsyncAPIWrapper:
