@@ -1,54 +1,108 @@
 import os
-from distutils.util import strtobool
 from types import ModuleType
 from typing import Any, Dict, List, Optional, Tuple, Union
 
 import firebolt.db as dbapi
 import sqlalchemy.types as sqltypes
-from firebolt.client.auth import Auth, ServiceAccount, UsernamePassword
+from firebolt.client.auth import (
+    Auth,
+    ClientCredentials,
+    FireboltCore,
+    UsernamePassword,
+)
 from firebolt.db import Cursor
 from sqlalchemy.engine import Connection as AlchemyConnection
 from sqlalchemy.engine import ExecutionContext, default
 from sqlalchemy.engine.url import URL
+from sqlalchemy.exc import ArgumentError
 from sqlalchemy.sql import compiler, text
 from sqlalchemy.types import (
+    ARRAY,
     BIGINT,
     BOOLEAN,
-    CHAR,
     DATE,
     DATETIME,
-    FLOAT,
     INTEGER,
+    NUMERIC,
+    REAL,
+    TEXT,
     TIMESTAMP,
-    VARCHAR,
 )
 
 
-class ARRAY(sqltypes.TypeEngine):
-    __visit_name__ = "ARRAY"
+class BYTEA(sqltypes.LargeBinary):
+    __visit_name__ = "BYTEA"
 
 
 # Firebolt data types compatibility with sqlalchemy.sql.types
 type_map = {
-    "char": CHAR,
-    "text": VARCHAR,
-    "varchar": VARCHAR,
-    "string": VARCHAR,
-    "float": FLOAT,
-    "double": FLOAT,
-    "double precision": FLOAT,
+    "text": TEXT,
+    "varchar": TEXT,
+    "string": TEXT,
+    "float": REAL,
+    "double": REAL,
+    "double precision": REAL,
+    "numeric": NUMERIC,
+    "decimal": NUMERIC,
+    "real": REAL,
     "boolean": BOOLEAN,
     "int": INTEGER,
     "integer": INTEGER,
     "bigint": BIGINT,
     "long": BIGINT,
     "timestamp": TIMESTAMP,
+    "timestamptz": TIMESTAMP,
+    "timestampntz": TIMESTAMP,
     "datetime": DATETIME,
     "date": DATE,
-    "array": ARRAY,
+    "bytea": BYTEA,
 }
 
 
+def resolve_type(fb_type: str) -> sqltypes.TypeEngine:
+    def removesuffix(s: str, suffix: str) -> str:
+        """Python < 3.9 compatibility"""
+        if s.endswith(suffix):
+            s = s[: -len(suffix)]
+        return s
+
+    result: sqltypes.TypeEngine
+    if fb_type.startswith("array"):
+        # Nested arrays not supported
+        dimensions = 0
+        while fb_type.startswith("array"):
+            dimensions += 1
+            fb_type = fb_type[6:-1]  # Strip ARRAY()
+            fb_type = removesuffix(removesuffix(fb_type, " not null"), " null")
+        result = ARRAY(resolve_type(fb_type), dimensions=dimensions)
+    else:
+        # Strip complex type info e.g. DECIMAL(8,23) -> DECIMAL
+        fb_type = fb_type[: fb_type.find("(")] if "(" in fb_type else fb_type
+        result = type_map.get(fb_type, DEFAULT_TYPE)  # type: ignore
+    return result
+
+
+def strtobool(val: str) -> int:
+    """Copy of deprecated distutils.util.strtobool
+    Convert a string representation of truth to true (1) or false (0).
+
+    True values are 'y', 'yes', 't', 'true', 'on', and '1'; false values
+    are 'n', 'no', 'f', 'false', 'off', and '0'. Values are compared
+    case-insensitively. Raises ValueError if
+    'val' is anything else.
+    """
+    val = val.lower()
+    if val in ("y", "yes", "t", "true", "on", "1"):
+        return 1
+    elif val in ("n", "no", "f", "false", "off", "0"):
+        return 0
+    else:
+        raise ValueError("invalid truth value %r" % (val,))
+
+
+DEFAULT_TYPE = TEXT
+
+
 class UniversalSet(set):
     def __contains__(self, item: Any) -> bool:
         return True
@@ -87,6 +141,9 @@ class FireboltDialect(default.DefaultDialect):
     supports_empty_insert = False
     supports_unicode_statements = True
     supports_unicode_binds = True
+    supports_statement_cache = False
+    # supports_statement_cache Set to False to disable warning
+    # https://sqlalche.me/e/20/cprf
     returns_unicode_strings = True
     description_encoding = None
     supports_native_boolean = True
@@ -99,42 +156,106 @@ class FireboltDialect(default.DefaultDialect):
         self.context: Union[ExecutionContext, Dict] = context or {}
 
     @classmethod
-    def dbapi(cls) -> ModuleType:
+    def import_dbapi(cls) -> ModuleType:  # For sqlalchemy >= 2.0.0
+        return dbapi
+
+    @classmethod
+    def dbapi(cls) -> ModuleType:  # Kept for backwards compatibility
         return dbapi
 
     def create_connect_args(self, url: URL) -> Tuple[List, Dict]:
         """
         Build firebolt-sdk compatible connection arguments.
-        URL format : firebolt://username:password@host:port/db_name
+        URL format : firebolt://id:secret@host:port/db_name
+        For Core: firebolt://db_name?url=http://localhost:8080
+        (full URL including scheme, host, port in url parameter)
         """
         parameters = dict(url.query)
-        # parameters are all passed as a string, we need to convert
-        # bool flag to boolean for SDK compatibility
-        token_cache_flag = bool(strtobool(parameters.pop("use_token_cache", "True")))
-        auth = (
-            ServiceAccount(url.username, url.password, token_cache_flag)
-            if "@" not in url.username
-            else UsernamePassword(url.username, url.password, token_cache_flag)
+        is_core_connection = "url" in parameters
+
+        if is_core_connection:
+            self._validate_core_connection(url, parameters)
+
+        token_cache_flag = self._parse_token_cache_flag(parameters)
+        auth = _determine_auth(url, token_cache_flag)
+        kwargs = self._build_connection_kwargs(
+            url, parameters, auth, is_core_connection
         )
+
+        return ([], kwargs)
+
+    def _validate_core_connection(self, url: URL, parameters: Dict[str, str]) -> None:
+        """Validate that Core connection parameters are correct.
+
+        Only validates credentials since FireboltCore auth handles other parameters.
+        """
+        if url.username or url.password:
+            raise ArgumentError(
+                "Core connections do not support username/password authentication"
+            )
+
+    def _parse_token_cache_flag(self, parameters: Dict[str, str]) -> bool:
+        """Parse and remove token cache flag from parameters."""
+        return bool(strtobool(parameters.pop("use_token_cache", "True")))
+
+    def _build_connection_kwargs(
+        self, url: URL, parameters: Dict[str, str], auth: Auth, is_core_connection: bool
+    ) -> Dict[str, Union[str, Auth, Dict[str, Any], None]]:
+        """Build connection kwargs for the SDK.
+
+        SQLAlchemy URL mapping:
+        - url.host -> database (Firebolt database name)
+        - url.database -> engine_name (Firebolt engine name)
+        """
         kwargs: Dict[str, Union[str, Auth, Dict[str, Any], None]] = {
             "database": url.host or None,
             "auth": auth,
             "engine_name": url.database,
             "additional_parameters": {},
         }
-        additional_parameters = {}
+
+        if is_core_connection:
+            kwargs["url"] = parameters.pop("url")
+
+        self._handle_account_name(parameters, auth, kwargs)
+        self._handle_environment_config(kwargs)
+        kwargs["additional_parameters"] = self._build_additional_parameters(parameters)
+        self._set_parameters = parameters
+
+        return kwargs
+
+    def _handle_account_name(
+        self,
+        parameters: Dict[str, str],
+        auth: Auth,
+        kwargs: Dict[str, Union[str, Auth, Dict[str, Any], None]],
+    ) -> None:
+        """Handle account_name parameter and validation."""
         if "account_name" in parameters:
             kwargs["account_name"] = parameters.pop("account_name")
-        self._set_parameters = parameters
-        # If URL override is not provided leave it to the sdk to determine the endpoint
+        elif isinstance(auth, ClientCredentials):
+            raise ArgumentError(
+                "account_name parameter must be provided to authenticate"
+            )
+
+    def _handle_environment_config(
+        self, kwargs: Dict[str, Union[str, Auth, Dict[str, Any], None]]
+    ) -> None:
+        """Handle environment-based configuration."""
         if "FIREBOLT_BASE_URL" in os.environ:
             kwargs["api_endpoint"] = os.environ["FIREBOLT_BASE_URL"]
-        # Tracking information
+
+    def _build_additional_parameters(
+        self, parameters: Dict[str, str]
+    ) -> Dict[str, Any]:
+        """Build additional parameters including tracking information."""
+        additional_parameters: Dict[str, Any] = {}
+
         if "user_clients" in parameters or "user_drivers" in parameters:
             additional_parameters["user_drivers"] = parameters.pop("user_drivers", [])
             additional_parameters["user_clients"] = parameters.pop("user_clients", [])
-        kwargs["additional_parameters"] = additional_parameters
-        return ([], kwargs)
+
+        return additional_parameters
 
     def get_schema_names(
         self, connection: AlchemyConnection, **kwargs: Any
@@ -148,6 +269,7 @@ class FireboltDialect(default.DefaultDialect):
         connection: AlchemyConnection,
         table_name: str,
         schema: Optional[str] = None,
+        **kw: Any
     ) -> bool:
         query = """
             select count(*) > 0 as exists_
@@ -211,7 +333,7 @@ class FireboltDialect(default.DefaultDialect):
         return [
             {
                 "name": row[0],
-                "type": type_map[row[1].lower()],
+                "type": resolve_type(row[1].lower()),
                 "nullable": get_is_nullable(row[2]),
                 "default": None,
             }
@@ -323,3 +445,15 @@ dialect = FireboltDialect
 
 def get_is_nullable(column_is_nullable: int) -> bool:
     return column_is_nullable == 1
+
+
+def _determine_auth(url: URL, token_cache_flag: bool = True) -> Auth:
+    parameters = dict(url.query)
+    is_core_connection = "url" in parameters
+
+    if is_core_connection:
+        return FireboltCore()
+    elif "@" in (url.username or ""):
+        return UsernamePassword(url.username, url.password, token_cache_flag)
+    else:
+        return ClientCredentials(url.username, url.password, token_cache_flag)
